//*****************************************************************************
//*****************************************************************************
//  FILENAME:   parameters.c
//  Version: 1.0, 2007/03/06
//  Generated by Andy Kiser
//
//  DESCRIPTION:  Handling of Volatile / Nonvolatile parameters
//
//	This library allows for the following maximum number of parameters
//		  8 Bit	: x16
//		 16 Bit	: x16
//		 32 Bit : x4
//		168 Bit : x9	(Strings, 20+1 characters)
//-----------------------------------------------------------------------------
//  Copyright (c) Toradex AG 2007. All Rights Reserved.
//*****************************************************************************
//*****************************************************************************
#define __DEBUG

#include <m8c.h>
#include <string.h>
#include "parameters.h"
#include "nonvolatile.h"
#include "sensor.h"

// Parameters in RAM
struct
{
	BYTE		Param_8   [NUM_PARAM_8];
	WORD		Param_16  [NUM_PARAM_16];
	DWORD		Param_32  [NUM_PARAM_32];
	tParam_168	Param_168 [NUM_PARAM_168];
} VolatileP;

// Parameters in Flash are stored in the area 0x3F00.0x3FFF
// make sure to reserve this area in the custom.lnk file !
// (-bNonvolatileP:0x3F00.0x3FFF)
#pragma abs_address:0x3F00
const struct
{
	BYTE		Param_8   [MAX_PARAM_8];
	WORD		Param_16  [MAX_PARAM_16];
	DWORD		Param_32  [MAX_PARAM_32];
	tParam_168	Param_168 [NUM_PARAM_168];
}	NonvolatileP = {{0,0},{500,500,5000},{0},{"UsrDevice","Frame","Ch1"}};
#pragma end_abs_address	


//-----------------------------------------------------------------------------
// copy memory area from Flash to RAM
// actually, I expected this function to be part of the <strings.h> library
// however it is not there, so I'm implementing it here
//-----------------------------------------------------------------------------
const void *cmemcpy(void *dest, const void *src, size_t n)
{
	BYTE *bdest = dest;
	const BYTE *bsrc  = src;

	while (n > 0)
	{
		*bdest = *bsrc;
		bdest++;
		bsrc++;
		n--;
	}
	
	return dest;
}

//-----------------------------------------------------------------------------
// Intialization on Powerup: Copy parameters from Flash to RAM
//-----------------------------------------------------------------------------
void Parameters_Start()
{
	cmemcpy(VolatileP.Param_8,   (const void*)NonvolatileP.Param_8,   sizeof(VolatileP.Param_8));
	cmemcpy(VolatileP.Param_16,  (const void*)NonvolatileP.Param_16,  sizeof(VolatileP.Param_16));
	cmemcpy(VolatileP.Param_32,  (const void*)NonvolatileP.Param_32,  sizeof(VolatileP.Param_32));
	cmemcpy(VolatileP.Param_168, (const void*)NonvolatileP.Param_168, sizeof(VolatileP.Param_168));
}

//-----------------------------------------------------------------------------
// Setting Parameters
//-----------------------------------------------------------------------------
void Set_Param_8 (BYTE index, BOOL NonVolatile, BYTE value)
{
	if (NonVolatile)
		Flash_Modify(&NonvolatileP.Param_8[index], &value, 1);
	VolatileP.Param_8[index] = value;
}

//-----------------------------------------------------------------------------
void Set_Param_16 (BYTE index, BOOL NonVolatile, WORD value)
{
	if (NonVolatile)
		Flash_Modify((const BYTE*)&NonvolatileP.Param_16[index], (BYTE*)&value, 2);
	VolatileP.Param_16[index] = value;
}

//-----------------------------------------------------------------------------
void Set_Param_32 (BYTE index, BOOL NonVolatile, DWORD value)
{
	if (NonVolatile)
		Flash_Modify((const BYTE*)&NonvolatileP.Param_32[index], (BYTE*)&value, 4);
	VolatileP.Param_32[index] = value;
}

//-----------------------------------------------------------------------------
void Set_Param_168 (BYTE index, BOOL NonVolatile, tParam_168 value)
{
	if (NonVolatile)
		Flash_Modify((const BYTE*)&NonvolatileP.Param_168[index], (BYTE*)value, 21);
	strncpy (VolatileP.Param_168[index], value, 21);

}

//-----------------------------------------------------------------------------
// Reading parameters
//-----------------------------------------------------------------------------
void Get_Param_8 (BYTE index, BOOL NonVolatile, BYTE *value)
{
	*value = NonVolatile ? NonvolatileP.Param_8[index] : VolatileP.Param_8[index];
}

//-----------------------------------------------------------------------------
void Get_Param_16 (BYTE index, BOOL NonVolatile, WORD *value)
{
	*value = NonVolatile ? NonvolatileP.Param_16[index] : VolatileP.Param_16[index];
}

//-----------------------------------------------------------------------------
void Get_Param_32 (BYTE index, BOOL NonVolatile, DWORD *value)
{
	*value = NonVolatile ? NonvolatileP.Param_32[index] : VolatileP.Param_32[index];
}

//-----------------------------------------------------------------------------
void Get_Param_168 (BYTE index, BOOL NonVolatile, tParam_168 value)
{
	if (NonVolatile)
		cstrncpy (value, NonvolatileP.Param_168[index], 21);
	else
		strncpy (value, VolatileP.Param_168[index], 21);
}
