//*****************************************************************************
//*****************************************************************************
//  FILENAME:   UsbSleepTimer.c
//  Version: 1.0, 2006/10/15
//  Generated by Andy Kiser
//
//  DESCRIPTION:  Detect USB inactivity and set "GotoSleep" Flag accordingly
//-----------------------------------------------------------------------------
//  Copyright (c) Toradex AG 2006. All Rights Reserved.
//*****************************************************************************
//*****************************************************************************
#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules

#define SUSPEND_TIMEOUT 2
BYTE SleepTimerCounts;

BOOL	GotoSleep;

//-----------------------------------------------------------------------------
// this function must be called AFTER USB has become active
// otherwise USB idle state would be detected and after 4ms the 
// device would go into suspend mode
//-----------------------------------------------------------------------------
void SleepTimer_Start()
{	
    OSC_CR0 = (OSC_CR0 & ~OSC_CR0_SLEEP) | OSC_CR0_SLEEP_512Hz;	// 1.95ms interrupt interval
	
	// enable SleeptTimer interrupt
	M8C_EnableIntMask( INT_MSK0, INT_MSK0_SLEEP );
	// reset timeout
	SleepTimerCounts = SUSPEND_TIMEOUT;	
	// initialize Flag
	GotoSleep = FALSE;
}

//-----------------------------------------------------------------------------
// sleep timer interrupt service routine
//-----------------------------------------------------------------------------
#pragma interrupt_handler SleepTimer_ISR
void SleepTimer_ISR ()
{
	if ( USB_bCheckActivity() )
		SleepTimerCounts = SUSPEND_TIMEOUT;
	else 
		if (SleepTimerCounts>0) 
			SleepTimerCounts--;
	
	GotoSleep = (SleepTimerCounts == 0);

	// clear SleepTimer interrupt flag
//	M8C_ClearIntFlag(INT_CLR0, INT_MSK0_SLEEP); 
	INT_CLR0 = ((BYTE)~INT_MSK0_SLEEP);
}